@@ -32,10 +32,9 @@
 
 #include <locale.h>
 
-#include <dbus/dbus.h>
-
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 
@@ -54,7 +53,10 @@ struct GsdBackgroundManagerPrivate
         GnomeBG     *bg;
         guint        timeout_id;
 
-        DBusConnection *dbus_connection;
+        GnomeBGCrossfade *fade;
+
+        GDBusProxy  *proxy;
+        guint        proxy_signal_id;
 };
 
 static void     gsd_background_manager_class_init  (GsdBackgroundManagerClass *klass);
@@ -62,13 +64,14 @@ static void     gsd_background_manager_init        (GsdBackgroundManager      *b
 static void     gsd_background_manager_finalize    (GObject             *object);
 
 static void setup_bg (GsdBackgroundManager *manager);
+static void connect_screen_signals (GsdBackgroundManager *manager);
 
 G_DEFINE_TYPE (GsdBackgroundManager, gsd_background_manager, G_TYPE_OBJECT)
 
 static gpointer manager_object = NULL;
 
 static gboolean
-nautilus_is_running (void)
+nautilus_is_drawing_background (GsdBackgroundManager *manager)
 {
        Atom           window_id_atom;
        Window         nautilus_xid;
@@ -77,10 +80,16 @@ nautilus_is_running (void)
        unsigned long  nitems;
        unsigned long  bytes_after;
        unsigned char *data;
-       int            retval;
        Atom           wmclass_atom;
        gboolean       running;
        gint           error;
+       gboolean       show_desktop_icons;
+
+       show_desktop_icons = g_settings_get_boolean (manager->priv->settings,
+                                                     "show-desktop-icons");
+       if (! show_desktop_icons) {
+               return FALSE;
+       }
 
        window_id_atom = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                      "NAUTILUS_DESKTOP_WINDOW_ID", True);
@@ -89,18 +98,18 @@ nautilus_is_running (void)
                return FALSE;
        }
 
-       retval = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                                    GDK_ROOT_WINDOW (),
-                                    window_id_atom,
-                                    0,
-                                    1,
-                                    False,
-                                    XA_WINDOW,
-                                    &actual_type,
-                                    &actual_format,
-                                    &nitems,
-                                    &bytes_after,
-                                    &data);
+       XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                           GDK_ROOT_WINDOW (),
+                           window_id_atom,
+                           0,
+                           1,
+                           False,
+                           XA_WINDOW,
+                           &actual_type,
+                           &actual_format,
+                           &nitems,
+                           &bytes_after,
+                           &data);
 
        if (data != NULL) {
                nautilus_xid = *(Window *) data;
@@ -120,18 +129,18 @@ nautilus_is_running (void)
 
        gdk_error_trap_push ();
 
-       retval = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                                    nautilus_xid,
-                                    wmclass_atom,
-                                    0,
-                                    24,
-                                    False,
-                                    XA_STRING,
-                                    &actual_type,
-                                    &actual_format,
-                                    &nitems,
-                                    &bytes_after,
-                                    &data);
+       XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                           nautilus_xid,
+                           wmclass_atom,
+                           0,
+                           24,
+                           False,
+                           XA_STRING,
+                           &actual_type,
+                           &actual_format,
+                           &nitems,
+                           &bytes_after,
+                           &data);
 
        error = gdk_error_trap_pop ();
 
@@ -158,6 +167,13 @@ nautilus_is_running (void)
        return running;
 }
 
+static void
+on_crossfade_finished (GsdBackgroundManager *manager)
+{
+        g_object_unref (manager->priv->fade);
+        manager->priv->fade = NULL;
+}
+
 static void
 draw_background (GsdBackgroundManager *manager,
                  gboolean              use_crossfade)
@@ -166,7 +182,8 @@ draw_background (GsdBackgroundManager *manager,
         int         n_screens;
         int         i;
 
-        if (nautilus_is_running ()) {
+
+        if (nautilus_is_drawing_background (manager)) {
                 return;
         }
 
@@ -191,11 +208,15 @@ draw_background (GsdBackgroundManager *manager,
                                                    TRUE);
 
                 if (use_crossfade) {
-                        GnomeBGCrossfade *fade;
 
-                        fade = gnome_bg_set_surface_as_root_with_crossfade (screen, surface);
-                        g_signal_connect (fade, "finished",
-                                          G_CALLBACK (g_object_unref), NULL);
+                        if (manager->priv->fade != NULL) {
+                                g_object_unref (manager->priv->fade);
+                        }
+
+                        manager->priv->fade = gnome_bg_set_surface_as_root_with_crossfade (screen, surface);
+                        g_signal_connect_swapped (manager->priv->fade, "finished",
+                                                  G_CALLBACK (on_crossfade_finished),
+                                                  manager);
                 } else {
                         gnome_bg_set_surface_as_root (screen, surface);
                 }
@@ -213,32 +234,14 @@ on_bg_transitioned (GnomeBG              *bg,
         draw_background (manager, FALSE);
 }
 
-static void
-background_changed (GsdBackgroundManager *manager,
-                    gboolean              use_crossfade)
-{
-        gboolean show_desktop_icons;
-
-        show_desktop_icons = g_settings_get_boolean (manager->priv->settings,
-                                                     "show-desktop-icons");
-
-        if (!nautilus_is_running () || !show_desktop_icons) {
-                if (manager->priv->bg == NULL) {
-                        setup_bg (manager);
-                }
-                gnome_bg_load_from_preferences (manager->priv->bg,
-                                                manager->priv->settings);
-                draw_background (manager, use_crossfade);
-        }
-}
-
 static gboolean
 settings_change_event_cb (GSettings            *settings,
                           gpointer              keys,
                           gint                  n_keys,
                           GsdBackgroundManager *manager)
 {
-        background_changed (manager, TRUE);
+        gnome_bg_load_from_preferences (manager->priv->bg,
+                                        manager->priv->settings);
         return FALSE;
 }
 
@@ -246,7 +249,7 @@ static void
 on_screen_size_changed (GdkScreen            *screen,
                         GsdBackgroundManager *manager)
 {
-        background_changed (manager, FALSE);
+        draw_background (manager, FALSE);
 }
 
 static void
@@ -258,6 +261,13 @@ watch_bg_preferences (GsdBackgroundManager *manager)
                           manager);
 }
 
+static void
+on_bg_changed (GnomeBG              *bg,
+               GsdBackgroundManager *manager)
+{
+        draw_background (manager, TRUE);
+}
+
 static void
 setup_bg (GsdBackgroundManager *manager)
 {
@@ -265,11 +275,17 @@ setup_bg (GsdBackgroundManager *manager)
 
         manager->priv->bg = gnome_bg_new ();
 
+        g_signal_connect (manager->priv->bg,
+                          "changed",
+                          G_CALLBACK (on_bg_changed),
+                          manager);
+
         g_signal_connect (manager->priv->bg,
                           "transitioned",
                           G_CALLBACK (on_bg_transitioned),
                           manager);
 
+        connect_screen_signals (manager);
         watch_bg_preferences (manager);
         gnome_bg_load_from_preferences (manager->priv->bg,
                                         manager->priv->settings);
@@ -279,61 +295,84 @@ static gboolean
 queue_draw_background (GsdBackgroundManager *manager)
 {
         manager->priv->timeout_id = 0;
-        if (nautilus_is_running ()) {
-                return FALSE;
-        }
+
         setup_bg (manager);
         draw_background (manager, FALSE);
+
         return FALSE;
 }
 
-static DBusHandlerResult
-on_bus_message (DBusConnection *connection,
-                DBusMessage    *message,
-                void           *user_data)
+static void
+queue_timeout (GsdBackgroundManager *manager)
 {
-        GsdBackgroundManager *manager = user_data;
+        if (manager->priv->timeout_id > 0)
+                return;
 
-        if (dbus_message_is_signal (message,
-                                    "org.gnome.SessionManager",
-                                    "SessionRunning")) {
-                /* If the session finishes then check if nautilus is
-                 * running and if not, set the background.
-                 *
-                 * We wait a few seconds after the session is up
-                 * because nautilus tells the session manager that its
-                 * ready before it sets the background.
-                 */
-                manager->priv->timeout_id = g_timeout_add_seconds (8,
-                                                                   (GSourceFunc)
-                                                                   queue_draw_background,
-                                                                   manager);
-                dbus_connection_remove_filter (connection,
-                                               on_bus_message,
-                                               manager);
+        /* If the session finishes then check if nautilus is
+         * running and if not, set the background.
+         *
+         * We wait a few seconds after the session is up
+         * because nautilus tells the session manager that its
+         * ready before it sets the background.
+         */
+        manager->priv->timeout_id = g_timeout_add_seconds (8,
+                                                           (GSourceFunc)
+                                                           queue_draw_background,
+                                                           manager);
+}
 
-                manager->priv->dbus_connection = NULL;
+static void
+disconnect_session_manager_listener (GsdBackgroundManager *manager)
+{
+        if (manager->priv->proxy && manager->priv->proxy_signal_id) {
+                g_signal_handler_disconnect (manager->priv->proxy,
+                                             manager->priv->proxy_signal_id);
+                manager->priv->proxy_signal_id = 0;
         }
+}
 
-        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+static void
+on_session_manager_signal (GDBusProxy   *proxy,
+                           const gchar  *sender_name,
+                           const gchar  *signal_name,
+                           GVariant     *parameters,
+                           gpointer      user_data)
+{
+        GsdBackgroundManager *manager = GSD_BACKGROUND_MANAGER (user_data);
+
+        if (g_strcmp0 (signal_name, "SessionRunning") == 0) {
+                queue_timeout (manager);
+                disconnect_session_manager_listener (manager);
+        }
 }
 
 static void
 draw_background_after_session_loads (GsdBackgroundManager *manager)
 {
-        DBusConnection *connection;
+        GError *error = NULL;
+        GDBusProxyFlags flags;
 
-        connection = dbus_bus_get (DBUS_BUS_SESSION, NULL);
-
-        if (connection == NULL) {
+        flags = G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
+                G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START;
+        manager->priv->proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                                              flags,
+                                                              NULL, /* GDBusInterfaceInfo */
+                                                              "org.gnome.SessionManager",
+                                                              "/org/gnome/SessionManager",
+                                                              "org.gnome.SessionManager",
+                                                              NULL, /* GCancellable */
+                                                              &error);
+        if (manager->priv->proxy == NULL) {
+                g_warning ("Could not listen to session manager: %s",
+                           error->message);
+                g_error_free (error);
                 return;
         }
 
-        if (!dbus_connection_add_filter (connection, on_bus_message, manager, NULL)) {
-                return;
-        };
-
-        manager->priv->dbus_connection = connection;
+        manager->priv->proxy_signal_id = g_signal_connect (manager->priv->proxy,
+                                                           "g-signal",
+                                                           G_CALLBACK (on_session_manager_signal),
+                                                           manager);
 }
 
 
@@ -407,8 +446,6 @@ gsd_background_manager_start (GsdBackgroundManager *manager,
                 draw_background_after_session_loads (manager);
         }
 
-        connect_screen_signals (manager);
-
         gnome_settings_profile_end (NULL);
 
         return TRUE;
@@ -423,10 +460,9 @@ gsd_background_manager_stop (GsdBackgroundManager *manager)
 
         disconnect_screen_signals (manager);
 
-        if (manager->priv->dbus_connection != NULL) {
-                dbus_connection_remove_filter (manager->priv->dbus_connection,
-                                               on_bus_message,
-                                               manager);
+        if (manager->priv->proxy) {
+                disconnect_session_manager_listener (manager);
+                g_object_unref (manager->priv->proxy);
         }
 
         g_signal_handlers_disconnect_by_func (manager->priv->settings,
@@ -455,10 +491,6 @@ gsd_background_manager_set_property (GObject        *object,
                                      const GValue   *value,
                                      GParamSpec     *pspec)
 {
-        GsdBackgroundManager *self;
-
-        self = GSD_BACKGROUND_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -472,10 +504,6 @@ gsd_background_manager_get_property (GObject        *object,
                                      GValue         *value,
                                      GParamSpec     *pspec)
 {
-        GsdBackgroundManager *self;
-
-        self = GSD_BACKGROUND_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -489,9 +517,6 @@ gsd_background_manager_constructor (GType                  type,
                                     GObjectConstructParam *construct_properties)
 {
         GsdBackgroundManager      *background_manager;
-        GsdBackgroundManagerClass *klass;
-
-        klass = GSD_BACKGROUND_MANAGER_CLASS (g_type_class_peek (GSD_TYPE_BACKGROUND_MANAGER));
 
         background_manager = GSD_BACKGROUND_MANAGER (G_OBJECT_CLASS (gsd_background_manager_parent_class)->constructor (type,
                                                                                                                         n_construct_properties,
@@ -503,10 +528,6 @@ gsd_background_manager_constructor (GType                  type,
 static void
 gsd_background_manager_dispose (GObject *object)
 {
-        GsdBackgroundManager *background_manager;
-
-        background_manager = GSD_BACKGROUND_MANAGER (object);
-
         G_OBJECT_CLASS (gsd_background_manager_parent_class)->dispose (object);
 }
 
