@@ -433,10 +433,6 @@ gsd_keyboard_manager_set_property (GObject        *object,
                                    const GValue   *value,
                                    GParamSpec     *pspec)
 {
-        GsdKeyboardManager *self;
-
-        self = GSD_KEYBOARD_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -450,10 +446,6 @@ gsd_keyboard_manager_get_property (GObject        *object,
                                    GValue         *value,
                                    GParamSpec     *pspec)
 {
-        GsdKeyboardManager *self;
-
-        self = GSD_KEYBOARD_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -467,9 +459,6 @@ gsd_keyboard_manager_constructor (GType                  type,
                                   GObjectConstructParam *construct_properties)
 {
         GsdKeyboardManager      *keyboard_manager;
-        GsdKeyboardManagerClass *klass;
-
-        klass = GSD_KEYBOARD_MANAGER_CLASS (g_type_class_peek (GSD_TYPE_KEYBOARD_MANAGER));
 
         keyboard_manager = GSD_KEYBOARD_MANAGER (G_OBJECT_CLASS (gsd_keyboard_manager_parent_class)->constructor (type,
                                                                                                       n_construct_properties,
@@ -481,10 +470,6 @@ gsd_keyboard_manager_constructor (GType                  type,
 static void
 gsd_keyboard_manager_dispose (GObject *object)
 {
-        GsdKeyboardManager *keyboard_manager;
-
-        keyboard_manager = GSD_KEYBOARD_MANAGER (object);
-
         G_OBJECT_CLASS (gsd_keyboard_manager_parent_class)->dispose (object);
 }
 
