@@ -103,10 +103,6 @@ gsd_mouse_manager_set_property (GObject        *object,
                                const GValue   *value,
                                GParamSpec     *pspec)
 {
-        GsdMouseManager *self;
-
-        self = GSD_MOUSE_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -120,10 +116,6 @@ gsd_mouse_manager_get_property (GObject        *object,
                                GValue         *value,
                                GParamSpec     *pspec)
 {
-        GsdMouseManager *self;
-
-        self = GSD_MOUSE_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -137,9 +129,6 @@ gsd_mouse_manager_constructor (GType                  type,
                               GObjectConstructParam *construct_properties)
 {
         GsdMouseManager      *mouse_manager;
-        GsdMouseManagerClass *klass;
-
-        klass = GSD_MOUSE_MANAGER_CLASS (g_type_class_peek (GSD_TYPE_MOUSE_MANAGER));
 
         mouse_manager = GSD_MOUSE_MANAGER (G_OBJECT_CLASS (gsd_mouse_manager_parent_class)->constructor (type,
                                                                                                       n_construct_properties,
@@ -151,10 +140,6 @@ gsd_mouse_manager_constructor (GType                  type,
 static void
 gsd_mouse_manager_dispose (GObject *object)
 {
-        GsdMouseManager *mouse_manager;
-
-        mouse_manager = GSD_MOUSE_MANAGER (object);
-
         G_OBJECT_CLASS (gsd_mouse_manager_parent_class)->dispose (object);
 }
 
@@ -555,7 +540,7 @@ set_disable_w_typing (GsdMouseManager *manager, gboolean state)
 {
         if (state && touchpad_is_present ()) {
                 GError *error = NULL;
-                char *args[5];
+                char *args[6];
 
                 if (manager->priv->syndaemon_spawned)
                         return 0;
@@ -563,8 +548,9 @@ set_disable_w_typing (GsdMouseManager *manager, gboolean state)
                 args[0] = "syndaemon";
                 args[1] = "-i";
                 args[2] = "2.0";
-                args[3] = "-k";
-                args[4] = NULL;
+                args[3] = "-K";
+                args[4] = "-R";
+                args[5] = NULL;
 
                 if (!g_find_program_in_path (args[0]))
                         return 0;
